@@ -1,13 +1,13 @@
 package Net::DNS::RR::NSEC3;
 
 #
-# $Id: NSEC3.pm 1382 2015-08-26 20:39:33Z willem $
+# $Id: NSEC3.pm 1686 2018-06-11 09:03:34Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1382 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1686 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR::NSEC);
 
 =head1 NAME
@@ -19,16 +19,14 @@ Net::DNS::RR::NSEC3 - DNS NSEC3 resource record
 
 use integer;
 
-use Carp;
-use MIME::Base32;
-
 use base qw(Exporter);
-use vars qw(@EXPORT_OK);
-@EXPORT_OK = qw(name2hash);
+our @EXPORT_OK = qw(name2hash);
+
+use Carp;
 
 require Net::DNS::DomainName;
 
-eval { require Digest::SHA };		## optional for simple Net::DNS RR
+eval 'require Digest::SHA';		## optional for simple Net::DNS RR
 
 my %digest = (
 	'1' => ['Digest::SHA', 1],				# RFC3658
@@ -39,19 +37,17 @@ my %digest = (
 		'SHA-1' => 1,					# RFC3658
 		);
 
-	my @digestbyalias = ( 'SHA' => 1 );
+	my @digestalias = ( 'SHA' => 1 );
 
 	my %digestbyval = reverse @digestbyname;
 
-	my @digestbynum = map { ( $_, 0 + $_ ) } keys %digestbyval;    # accept algorithm number
-
-	my %digestbyname = map { s /[^A-Za-z0-9]//g; $_ } @digestbyalias, @digestbyname, @digestbynum;
-
+	my @digestrehash = map /^\d/ ? ($_) x 3 : do { s/[\W_]//g; uc($_) }, @digestbyname;
+	my %digestbyname = ( @digestalias, @digestrehash );	# work around broken cperl
 
 	sub _digestbyname {
 		my $name = shift;
 		my $key	 = uc $name;				# synthetic key
-		$key =~ s /[^A-Z0-9]//g;			# strip non-alphanumerics
+		$key =~ s /[\W_]//g;				# strip non-alphanumerics
 		$digestbyname{$key} || croak "unknown digest type $name";
 	}
 
@@ -70,20 +66,19 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $ssize = unpack "\@$offset x4 C", $$data;
 	@{$self}{qw(algorithm flags iterations saltbin)} = unpack "\@$offset CCnx a$ssize", $$data;
 	$offset += 5 + $ssize;
-	my $hsize = unpack "\@$offset C",	  $$data;
-	my $hname = unpack "\@$offset x a$hsize", $$data;
-	$self->hnxtname( MIME::Base32::encode_09AV $hname, '' );
+	my $hsize = unpack "\@$offset C", $$data;
+	$self->{hnxtname} = unpack "\@$offset x a$hsize", $$data;
 	$offset += 1 + $hsize;
 	$self->{typebm} = substr $$data, $offset, ( $limit - $offset );
+	$self->{hashfn} = _hashfn( @{$self}{qw(algorithm iterations saltbin)} );
 }
 
 
 sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	return '' unless $self->{typebm};
 	my $salt = $self->saltbin;
-	my $hash = MIME::Base32::decode_09AV uc( $self->hnxtname );
+	my $hash = $self->{hnxtname};
 	pack 'CCn C a* C a* a*', $self->algorithm, $self->flags, $self->iterations,
 			length($salt), $salt,
 			length($hash), $hash,
@@ -94,11 +89,10 @@ sub _encode_rdata {			## encode rdata as wire-format octet string
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{hnxtname};
-	my @rdata = $self->algorithm, $self->flags, $self->iterations,
-			$self->salt || '-',
-			$self->hnxtname,
-			$self->typelist;
+	my @rdata = (
+		$self->algorithm, $self->flags, $self->iterations,
+		$self->salt || '-', $self->hnxtname, $self->typelist
+		);
 }
 
 
@@ -112,6 +106,7 @@ sub _parse_rdata {			## populate RR from rdata in argument list
 	$self->salt($salt) unless $salt eq '-';
 	$self->hnxtname(shift);
 	$self->typelist(@_);
+	$self->{hashfn} = _hashfn( @{$self}{qw(algorithm iterations saltbin)} );
 }
 
 
@@ -148,8 +143,7 @@ sub optout {
 	my $bit = 0x01;
 	for ( shift->{flags} ) {
 		my $set = $bit | ( $_ ||= 0 );
-		return $bit & $_ unless scalar @_;
-		$_ = (shift) ? $set : ( $set ^ $bit );
+		$_ = (shift) ? $set : ( $set ^ $bit ) if scalar @_;
 		return $_ & $bit;
 	}
 }
@@ -165,9 +159,8 @@ sub iterations {
 
 sub salt {
 	my $self = shift;
-
-	$self->saltbin( pack "H*", map { die "!hex!" if m/[^0-9A-Fa-f]/; $_ } join "", @_ ) if scalar @_;
-	unpack "H*", $self->saltbin() if defined wantarray;
+	return unpack "H*", $self->saltbin() unless scalar @_;
+	$self->saltbin( pack "H*", map /[^\dA-F]/i ? croak "corrupt hex" : $_, join "", @_ );
 }
 
 
@@ -181,73 +174,139 @@ sub saltbin {
 
 sub hnxtname {
 	my $self = shift;
-	return $self->{hnxtname} unless scalar @_;
-	$self->{hnxtname} = lc( shift || '' );
+	$self->{hnxtname} = _decode_base32hex(shift) if scalar @_;
+	_encode_base32hex( $self->{hnxtname} ) if defined wantarray;
 }
 
 
 sub covered {
-	my $self = shift;
-	my $name = shift;
+	my ( $self, $name ) = @_;
 
-	# first test if the domain name is in the NSEC3 zone.
-	my @domainlabels = new Net::DNS::DomainName($name)->_wire;
-	my ( $ownerhash, @zonelabels ) = map lc($_), $self->{owner}->_wire;
+	my ( $owner, @zone ) = $self->{owner}->_wire;
+	my $ownerhash = _decode_base32hex($owner);
+	my $nexthash  = $self->{hnxtname};
 
-	foreach ( reverse @zonelabels ) {
-		return 0 unless $_ eq lc( pop(@domainlabels) || return 0 );
+	my @label = new Net::DNS::DomainName($name)->_wire;
+	my @close = @label;
+	foreach (@zone) { pop(@close) }				# strip zone labels
+	return if lc($name) ne lc( join '.', @close, @zone );	# out of zone
+
+	my $hashfn = $self->{hashfn};
+
+	foreach (@close) {
+		my $hash = &$hashfn( join '.', @label );
+		my $cmp1 = $hash cmp $ownerhash;
+		last unless $cmp1;				# stop at provable encloser
+		return 1 if ( $cmp1 + ( $nexthash cmp $hash ) ) == 2;
+		shift @label;
 	}
-
-	my $namehash = name2hash( $self->algorithm, $name, $self->iterations, $self->saltbin );
-	my $nexthash = $self->hnxtname;
-
-	unless ( $ownerhash lt $nexthash ) {			# last or only NSEC3 RR
-		return 1 if $namehash lt $nexthash;
-		return $namehash gt $ownerhash;
-	}
-
-	return 0 unless $namehash gt $ownerhash;		# general case
-	return $namehash lt $nexthash;
+	return;
 }
 
 
 sub match {
-	my $self = shift;
-	my $name = shift;
+	my ( $self, $name ) = @_;
 
-	my ($ownerhash) = $self->{owner}->_wire;
-	my $namehash = name2hash( $self->algorithm, $name, $self->iterations, $self->saltbin );
+	my ($owner) = $self->{owner}->_wire;
+	my $ownerhash = _decode_base32hex($owner);
 
-	$namehash eq lc($ownerhash);
+	my $hashfn = $self->{hashfn};
+	$ownerhash eq &$hashfn($name);
 }
 
 
+sub encloser {
+	my ( $self, $qname ) = @_;
+
+	my ( $owner, @zone ) = $self->{owner}->_wire;
+	my $ownerhash = _decode_base32hex($owner);
+	my $nexthash  = $self->{hnxtname};
+
+	my @label = new Net::DNS::DomainName($qname)->_wire;
+	my @close = @label;
+	foreach (@zone) { pop(@close) }				# strip zone labels
+	return if lc($qname) ne lc( join '.', @close, @zone );	# out of zone
+
+	my $hashfn = $self->{hashfn};
+
+	my $encloser = $qname;
+	shift @label;
+	foreach (@close) {
+		my $nextcloser = $encloser;
+		my $hash = &$hashfn( $encloser = join '.', @label );
+		shift @label;
+		next if $hash ne $ownerhash;
+		$self->{nextcloser} = $nextcloser;		# next closer name
+		return $encloser;				# closest provable encloser
+	}
+	return;
+}
+
+
+sub nextcloser { return shift->{nextcloser} }
+
+
 ########################################
 
-sub hashalgo { &algorithm; }					# uncoverable pod
+sub _decode_base32hex {
+	local $_ = shift || '';
+	tr [0-9A-Va-v\060-\071\101-\126\141-\166] [\000-\037\012-\037\000-\037\012-\037];
+	my $l = ( 5 * length ) & ~7;
+	pack "B$l", join '', map unpack( 'x3a5', unpack 'B8', $_ ), split //;
+}
 
 
-sub name2hash {
-	my $hashalg    = shift;					# uncoverable pod
-	my $name       = shift;
+sub _encode_base32hex {
+	my @split = grep length, split /(\S{5})/, unpack 'B*', shift;
+	local $_ = join '', map pack( 'B*', "000$_" ), @split;
+	tr [\000-\037] [0-9a-v];
+	return $_;
+}
+
+
+my ( $cache1, $cache2, $limit ) = ( {}, {}, 10 );
+
+sub _hashfn {
+	my $hashalg    = shift;
 	my $iterations = shift;
 	my $salt       = shift || '';
 
 	my $arglist = $digest{$hashalg};
-	my ( $object, @argument ) = @$arglist;
-	my $hash = $object->new(@argument);
+	my ( $class, @argument ) = @$arglist;
+	my $instance = $class->new(@argument);
 
-	my $wirename = new Net::DNS::DomainName2535($name)->encode;
 	$iterations++;
+	my $key_adjunct = pack 'Cna*', $hashalg, $iterations, $salt;
 
-	while ( $iterations-- ) {
-		$hash->add($wirename);
-		$hash->add($salt);
-		$wirename = $hash->digest;
-	}
+	return sub {
+		my $name  = new Net::DNS::DomainName(shift)->canonical;
+		my $key	  = join '', $name, $key_adjunct;
+		my $cache = $$cache1{$key} ||= $$cache2{$key};	# two layer cache
+		return $cache if defined $cache;
+		( $cache1, $cache2, $limit ) = ( {}, $cache1, 50 ) unless $limit--;    # recycle cache
 
-	local $_;						# CPAN RT#16528
-	return lc MIME::Base32::encode_09AV( $wirename, '' );	# per RFC4648, 7.
+		my $hash = $name;
+		my $iter = $iterations;
+		$instance->reset;
+		while ( $iter-- ) {
+			$instance->add($hash);
+			$instance->add($salt);
+			$hash = $instance->digest;
+		}
+		return $$cache1{$key} = $hash;
+	};
+}
+
+
+sub hashalgo { &algorithm; }					# uncoverable pod
+
+sub name2hash {
+	my $hashalg    = shift;					# uncoverable pod
+	my $name       = shift;
+	my $iterations = shift || 0;
+	my $salt       = pack 'H*', shift || '';
+	my $hash       = _hashfn( $hashalg, $iterations, $salt );
+	_encode_base32hex( &$hash($name) );
 }
 
 
@@ -360,29 +419,34 @@ interpolated into a string.
 
 =head2 covered, match
 
-    print "covered" if $rr->covered{'example.foo'}
+    print "covered" if $rr->covered( 'example.foo' );
 
-covered() returns a nonzero value when the the domain name provided as argument
-is covered as defined in the NSEC3 specification:
+covered() returns a Boolean true value if the hash of the domain name
+argument, or ancestor of that name, falls between the owner name and
+the next hashed owner name of the NSEC3 RR.
 
-   To cover:  An NSEC3 RR is said to "cover" a name if the hash of the
-      name or "next closer" name falls between the owner name and the
-      next hashed owner name of the NSEC3.  In other words, if it proves
-      the nonexistence of the name, either directly or by proving the
-      nonexistence of an ancestor of the name.
+Similarly match() returns a Boolean true value if the hash of the
+domain name argument matches the owner name of the NSEC3 RR.
 
+=head2 encloser, nextcloser
 
-Similarly matched() returns a nonzero value when the domainname in the argument
-matches as defined in the NSEC3 specification:
+    $encloser = $rr->encloser( 'example.foo' );
+    print "encloser: $encloser\n" if $encloser;
 
-   To match: An NSEC3 RR is said to "match" a name if the owner name
-      of the NSEC3 RR is the same as the hashed owner name of that
-      name.
+encloser() returns the name of the closest provable encloser of the
+query name argument if obtainable from the specified NSEC3 RRi, and
+otherwise undefined.
+
+nextcloser() returns the next closer name, which is one label longer
+than the closest encloser.  This is only valid after encloser() has
+returned a valid domain name.
 
 
 =head1 COPYRIGHT
 
-Copyright (c)2007,2008 NLnet Labs.  Author Olaf M. Kolkman
+Copyright (c)2017,2018 Dick Franks
+
+Portions Copyright (c)2007,2008 NLnet Labs.  Author Olaf M. Kolkman
 
 All rights reserved.
 
