@@ -1,13 +1,13 @@
 package Net::DNS::RR::RRSIG;
 
 #
-# $Id: RRSIG.pm 1408 2015-10-06 20:35:56Z willem $
+# $Id: RRSIG.pm 1686 2018-06-11 09:03:34Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1408 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1686 $)[1];
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::RR);
 
 =head1 NAME
@@ -27,17 +27,25 @@ use Net::DNS::Parameters;
 
 use constant DEBUG => 0;
 
-use constant UTIL => defined eval { require Scalar::Util; };
+use constant UTIL => defined eval 'use Scalar::Util 1.25; 1;';
 
-use constant PRIVATE => defined eval { require Net::DNS::SEC::Private; };
+use constant PRIVATE => defined eval 'require Net::DNS::SEC::Private';
 
-use constant DSA => scalar eval { require Net::DNS::SEC::DSA; };
-use constant RSA => scalar eval { require Net::DNS::SEC::RSA; };
+use constant DSA => defined eval 'require Net::DNS::SEC::DSA';
+use constant RSA => defined eval 'require Net::DNS::SEC::RSA';
 
-use constant ECDSA => scalar eval { require Net::DNS::SEC::ECDSA; };
-use constant GOST  => scalar eval { require Net::DNS::SEC::ECCGOST; };
+use constant ECDSA => defined eval 'require Net::DNS::SEC::ECDSA';
+use constant EdDSA => defined eval 'require Net::DNS::SEC::EdDSA';
+use constant GOST  => defined eval 'require Digest::GOST; require Net::DNS::SEC::ECCGOST';
 
-use constant DNSSEC => PRIVATE && ( RSA || DSA || ECDSA || GOST );
+use constant DNSSEC => PRIVATE && ( RSA || DSA || ECDSA || EdDSA );
+
+my $DSA = DSA ? 'Net::DNS::SEC::DSA' : 0;
+my $RSA = RSA ? 'Net::DNS::SEC::RSA' : 0;
+
+my $ECDSA = ECDSA ? 'Net::DNS::SEC::ECDSA'   : 0;
+my $EdDSA = EdDSA ? 'Net::DNS::SEC::EdDSA'   : 0;
+my $GOST  = GOST  ? 'Net::DNS::SEC::ECCGOST' : 0;
 
 my @field = qw(typecovered algorithm labels orgttl sigexpiration siginception keytag);
 
@@ -56,7 +64,7 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	my $signame = $self->{signame} || return '';
+	my $signame = $self->{signame};
 	pack 'n C2 N3 n a* a*', @{$self}{@field}, $signame->canonical, $self->sigbin;
 }
 
@@ -64,9 +72,9 @@ sub _encode_rdata {			## encode rdata as wire-format octet string
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	my $signame = $self->{signame} || return '';
-	my @sig64 = split /\s+/, encode_base64( $self->sigbin );
-	my @rdata = ( map( $self->$_, @field ), $signame->string, @sig64 );
+	my $signame = $self->{signame};
+	my @sig64   = split /\s+/, encode_base64( $self->sigbin );
+	my @rdata   = ( map( $self->$_, @field ), $signame->string, @sig64 );
 }
 
 
@@ -89,7 +97,8 @@ sub _defaults {				## specify RR attribute default values
 # source: http://www.iana.org/assignments/dns-sec-alg-numbers
 #
 {
-	my @algbyname = (		## Reserved	=> 0,	# [RFC4034][RFC4398]
+	my @algbyname = (
+		'DELETE'	     => 0,			# [RFC4034][RFC4398][RFC8078]
 		'RSAMD5'	     => 1,			# [RFC3110][RFC4034]
 		'DH'		     => 2,			# [RFC2539]
 		'DSA'		     => 3,			# [RFC3755][RFC2536]
@@ -104,6 +113,8 @@ sub _defaults {				## specify RR attribute default values
 		'ECC-GOST'	     => 12,			# [RFC5933]
 		'ECDSAP256SHA256'    => 13,			# [RFC6605]
 		'ECDSAP384SHA384'    => 14,			# [RFC6605]
+		'ED25519'	     => 15,			# [RFC8080]
+		'ED448'		     => 16,			# [RFC8080]
 
 		'INDIRECT'   => 252,				# [RFC4034]
 		'PRIVATEDNS' => 253,				# [RFC4034]
@@ -113,21 +124,16 @@ sub _defaults {				## specify RR attribute default values
 
 	my %algbyval = reverse @algbyname;
 
-	my $map = sub {
-		my $arg = shift;
-		return $arg if $arg =~ /^\d/;
-		$arg =~ s/[^A-Za-z0-9]//g;			# strip non-alphanumerics
-		uc($arg);
-	};
-
-	my @pairedval = sort ( 1 .. 254, 1 .. 254 );		# also accept number
-	my %algbyname = map &$map($_), @algbyname, @pairedval;
+	my @algrehash = map /^\d/ ? ($_) x 3 : do { s/[\W_]//g; uc($_) }, @algbyname;
+	my %algbyname = @algrehash;    # work around broken cperl
 
 	sub _algbyname {
-		my $name = shift;
-		my $key	 = uc $name;				# synthetic key
-		$key =~ s/[^A-Z0-9]//g;				# strip non-alphanumerics
-		$algbyname{$key} || croak "unknown algorithm $name";
+		my $arg = shift;
+		my $key = uc $arg;				# synthetic key
+		$key =~ s/[\W_]//g;				# strip non-alphanumerics
+		my $val = $algbyname{$key};
+		return $val if defined $val;
+		return $key =~ /^\d/ ? $arg : croak "unknown algorithm $arg";
 	}
 
 	sub _algbyval {
@@ -137,22 +143,23 @@ sub _defaults {				## specify RR attribute default values
 }
 
 
-my $RSA	  = RSA	  ? 'Net::DNS::SEC::RSA'     : 0;
-my $DSA	  = DSA	  ? 'Net::DNS::SEC::DSA'     : 0;
-my $ECDSA = ECDSA ? 'Net::DNS::SEC::ECDSA'   : 0;
-my $GOST  = GOST  ? 'Net::DNS::SEC::ECCGOST' : 0;
-
-my %SEC = (
-	1  => $RSA,
-	3  => $DSA,
+my %DNSSEC_sign = (
 	5  => $RSA,
-	6  => $DSA,
 	7  => $RSA,
 	8  => $RSA,
 	10 => $RSA,
-	12 => $GOST,
 	13 => $ECDSA,
 	14 => $ECDSA,
+	15 => $EdDSA,
+	16 => $EdDSA,
+	);
+
+my %DNSSEC_verify = (
+	1  => $RSA,
+	3  => $DSA,
+	6  => $DSA,
+	12 => $GOST,
+	%DNSSEC_sign,
 	);
 
 
@@ -239,9 +246,8 @@ sub signame {
 
 sub sig {
 	my $self = shift;
-
-	$self->sigbin( MIME::Base64::decode( join "", @_ ) ) if scalar @_;
-	MIME::Base64::encode( $self->sigbin(), "" ) if defined wantarray;
+	return MIME::Base64::encode( $self->sigbin(), "" ) unless scalar @_;
+	$self->sigbin( MIME::Base64::decode( join "", @_ ) );
 }
 
 
@@ -406,7 +412,7 @@ sub _ordered($$) {			## irreflexive 32-bit partial ordering
 	return defined $b unless defined $a;			# ( undef, any )
 	return 0 unless defined $b;				# ( any, undef )
 
-	# unwise to assume 32-bit arithmetic, or that integer overflow goes unpunished
+	# unwise to assume 64-bit arithmetic, or that 32-bit integer overflow goes unpunished
 	if ( $a < 0 ) {						# translate $a<0 region
 		$a = ( $a ^ 0x80000000 ) & 0xFFFFFFFF;		#  0	 <= $a < 2**31
 		$b = ( $b ^ 0x80000000 ) & 0xFFFFFFFF;		# -2**31 <= $b < 2**32
@@ -430,7 +436,7 @@ sub _string2time {			## parse time specification string
 	croak 'undefined time' unless defined $arg;
 	return int($arg) if length($arg) < 12;
 	my ( $y, $m, @dhms ) = unpack 'a4 a2 a2 a2 a2 a2', $arg . '00';
-	unless ( $arg gt '20380119031407' ) {			# calendar folding
+	if ( $arg lt '20380119031408' ) {			# calendar folding
 		return timegm( reverse(@dhms), $m - 1, $y ) if $y < 2026;
 		return timegm( reverse(@dhms), $m - 1, $y - 56 ) + $y2026;
 	} elsif ( $y > 2082 ) {
@@ -548,7 +554,7 @@ sub _CreateSig {
 		my $self = shift;
 
 		my $algorithm = $self->algorithm;
-		my $class     = $SEC{$algorithm};
+		my $class     = $DNSSEC_sign{$algorithm};
 
 		eval {
 			die "algorithm $algorithm not supported" unless $class;
@@ -563,7 +569,7 @@ sub _VerifySig {
 		my $self = shift;
 
 		my $algorithm = $self->algorithm;
-		my $class     = $SEC{$algorithm};
+		my $class     = $DNSSEC_verify{$algorithm};
 
 		my $retval = eval {
 			die "algorithm $algorithm not supported" unless $class;
@@ -597,8 +603,8 @@ __END__
 
     use Net::DNS::SEC;
     $sigrr = create Net::DNS::RR::RRSIG( \@rrset, $keypath,
-					sigex => 20151231010101
-					sigin => 20151201010101
+					sigex => 20171231010101
+					sigin => 20171201010101
 					);
 
     $sigrr->verify( \@rrset, $keyrr ) || die $sigrr->vrfyerrstr;
@@ -722,8 +728,8 @@ Create a signature over a RR set.
     $sigrr = create Net::DNS::RR::RRSIG( \@rrsetref, $keypath );
 
     $sigrr = create Net::DNS::RR::RRSIG( \@rrsetref, $keypath,
-					sigex => 20151231010101
-					sigin => 20151201010101
+					sigex => 20171231010101
+					sigin => 20171201010101
 					);
     $sigrr->print;
 
@@ -749,8 +755,8 @@ containing the private key as generated by dnssec-keygen.
 The optional remaining arguments consist of ( name => value ) pairs
 as follows:
 
-	sigex  => 20151231010101,	# signature expiration
-	sigin  => 20151201010101,	# signature inception
+	sigex  => 20171231010101,	# signature expiration
+	sigin  => 20171201010101,	# signature inception
 	sigval => 30,			# validity window (days)
 	ttl    => 3600			# TTL
 
@@ -811,17 +817,20 @@ The code is not optimised for speed.
 It is probably not suitable to be used for signing large zones.
 
 If this code is still around in 2100 (not a leap year) you will
-need to check for proper handling of times ...
+need to check for proper handling of times after 28th February.
 
 =head1 ACKNOWLEDGMENTS
 
-Andy Vaskys (Network Associates Laboratories) supplied the code for
-handling RSA with SHA1 (Algorithm 5).
+Although their original code may have disappeared following redesign of
+Net::DNS, Net::DNS::SEC and the OpenSSL API, the following individual
+contributors deserve to be recognised for their significant influence
+on the development of the RRSIG package.
 
-T.J. Mather, the Crypt::OpenSSL::DSA maintainer, for his quick
-responses to bug report and feature requests.
+Andy Vaskys (Network Associates Laboratories) supplied code for RSA.
 
-Dick Franks added support for elliptic curve signatures.
+T.J. Mather provided support for the DSA algorithm.
+
+Dick Franks added support for elliptic curve and Edwards curve algorithms.
 
 Mike McCauley created the Crypt::OpenSSL::ECDSA perl extension module
 specifically for this development.
@@ -864,8 +873,8 @@ DEALINGS IN THE SOFTWARE.
 L<perl>, L<Net::DNS>, L<Net::DNS::RR>, L<Net::DNS::SEC>,
 RFC4034, RFC6840, RFC3755,
 L<Net::DNS::SEC::DSA>,
-L<Net::DNS::SEC::ECCGOST>,
 L<Net::DNS::SEC::ECDSA>,
+L<Net::DNS::SEC::EdDSA>,
 L<Net::DNS::SEC::RSA>
 
 L<Algorithm Numbers|http://www.iana.org/assignments/dns-sec-alg-numbers>
